package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesConfigService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesNcProcessingService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.dao.IMesNcProcessingDao;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesNcProcessingInputModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesNcProcessingModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesNcProcessingPartAndLotModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesNcProcessingViewModel;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.ResultBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-05-10 15:59
 * @Modify:
 **/
@Service
@Slf4j
public class MesNcProcessingService implements IMesNcProcessingService {

    @Autowired
    private MesDefectRecordRepository defectRecordRepository;

    @Autowired
    private MesPartRepository partRepository;

    @Autowired
    private MesPartTypePictureRepository partTypePictureRepository;

    @Autowired
    private MesPartTypeDefectRepository partTypeDefectRepository;

    @Autowired
    private MesLocationConfigRepository locationConfigRepository;

    @Autowired
    private IMesConfigService configService;

    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private MesMoveRepository moveRepository;

    @Autowired
    private IMesNcProcessingDao ncProcessingDao;

    @Override
    public MesNcProcessingModel queryProduceSn(String serialNumber, String partNo, String lot, String org) {


        if (!StringUtils.isEmpty(serialNumber)) {
            //检查不良条码
            List<MesDefectRecord> defectRecordList = queryDefectRecordBySerialNumber(serialNumber, org);

            //检查物料
            MesPart part = checkPart(defectRecordList, org);

            //构造返回结果
            MesNcProcessingModel model = getModel(part, defectRecordList, org);
            return model;
        } else {

            //检查不良条码
            List<MesDefectRecord> defectRecordList = queryDefectRecordByPartAndLot(partNo, lot, org);

            //检查物料
            MesPart part = checkPartByPartNo(partNo, org);

            //构造返回结果
            MesNcProcessingModel model = getModel(part, defectRecordList, org);
            return model;
        }
    }

    @Override
    public ListPager<MesNcProcessingPartAndLotModel> queryPartLot(String partNo, String lot, String org, Pager pager) {

        pager = PagerHelper.getPager(pager, ncProcessingDao.queryDefectRecordByPagerCount(partNo, lot, org));

        List<MesNcProcessingPartAndLotModel> modelList = ncProcessingDao.queryDefectRecordByPager(partNo, lot, org, pager);

        return new ListPager(modelList, pager);
    }

    @Override
    public void saveNc(MesNcProcessingInputModel model, String org) {

        if (!StringUtils.isEmpty(model.getSerialNumber())) {
            String serialNumber = model.getSerialNumber();
            Integer type = model.getType();
            Integer person = model.getPerson();

            //检查不良条码
            List<MesDefectRecord> defectRecordList = queryDefectRecordBySerialNumber(serialNumber, org);

            //检查物料
            MesPart part = checkPartByPartNo(defectRecordList.get(0).getPartNo(), org);

            //保存数据
            saveDate(part, defectRecordList, type, person, org);
        } else {

            String partNo = model.getPartNo();
            String lot = model.getLot();
            Integer type = model.getType();
            Integer person = model.getPerson();

            //检查不良条码
            List<MesDefectRecord> defectRecordList = queryDefectRecordByPartAndLot(partNo, lot, org);

            //检查物料
            MesPart part = checkPartByPartNo(partNo, org);

            //保存数据
            saveDate(part, defectRecordList, type, person, org);
        }
    }

    private MesPart checkPart(List<MesDefectRecord> defectRecordList, String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(defectRecordList.get(0).getPartNo(), "partNo", packBean);
        MesPart part = partRepository.getByProperty(packBean);

        //若不存在则提示物料信息不存在
        if (StringUtils.isEmpty(part)) {
            throw new ImppBusiException(String.format("【%s】此条码对应【%s】物料号不存在，请检查数据", defectRecordList.get(0).getSerialNumber(), defectRecordList.get(0).getPartNo()));
        }

        //根据条码查询零件类型（partType）
        if (StringUtils.isEmpty(part.getPartTypeCode())) {
            throw new ImppBusiException(String.format("【%s】此条码对应【%s】物料号未维护零件类型，请检查数据", defectRecordList.get(0).getSerialNumber(), defectRecordList.get(0).getPartNo()));
        }


        return part;
    }

    private MesPart checkPartByPartNo(String partNo, String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", packBean);
        MesPart part = partRepository.getByProperty(packBean);

        //若不存在则提示物料信息不存在
        if (StringUtils.isEmpty(part)) {
            throw new ImppBusiException(String.format("【%s】物料号不存在，请检查数据", partNo));
        }

        return part;
    }


    private List<MesDefectRecord> queryDefectRecordBySerialNumber(String serialNumber, String org) {

        //查询扫描的条码
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(serialNumber, "serialNumber", packBean);
//        DdlPreparedPack.getNumEqualPack(0, "ncType", packBean);
        List<MesDefectRecord> defectRecordList = defectRecordRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(defectRecordList)) {
            throw new ImppBusiException(String.format("【%s】此条码不存在可疑品记录，请检查数据", serialNumber));
        }

        defectRecordList.forEach(k -> {
            if (k.getNcType() != 0) {
                throw new ImppBusiException(String.format("【%s】此条码已经过NC处理，请检查数据", serialNumber));
            }
        });


        return defectRecordList;
    }

    private List<MesDefectRecord> queryDefectRecordByPartAndLot(String partNo, String Lot, String org) {

        //查询扫描的条码
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(Lot, "lot", packBean);
//        DdlPreparedPack.getNumEqualPack(0, "ncType", packBean);
        List<MesDefectRecord> defectRecordList = defectRecordRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(defectRecordList)) {
            throw new ImppBusiException(String.format("【%s】此物料号+【%s】批次不存在可疑品记录，请检查数据", partNo, Lot));
        }

        defectRecordList.forEach(k -> {
            if (k.getNcType() != 0) {
                throw new ImppBusiException(String.format("【%s】此物料号+【%s】批次已经过NC处理，请检查数据", partNo, Lot));
            }
        });


        return defectRecordList;
    }

    private MesPartTypePicture checkPartTypePicture(MesPart part, String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(part.getPartTypeCode(), "partTypeCode", packBean);
        MesPartTypePicture partTypePicture = partTypePictureRepository.getByProperty(packBean);

        //若不存在则提示零件类别不存在
        if (StringUtils.isEmpty(partTypePicture)) {
            throw new ImppBusiException(String.format("【%s】物料号对应的【%s】零件类别不存在，请检查数据", part.getPartNo(), part.getPartTypeCode()));
        }
        //若不存在则提示未维护正面图片
        if (StringUtils.isEmpty(partTypePicture.getFrontPictureName())) {
            throw new ImppBusiException(String.format("【%s】物料号对应的【%s】零件类别未维护正面图片，请检查数据", part.getPartNo(), part.getPartTypeCode()));
        }
        //若不存在则提示未维护反面图片
        if (StringUtils.isEmpty(partTypePicture.getBackPictureName())) {
            throw new ImppBusiException(String.format("【%s】物料号对应的【%s】零件类别未维护反面图片，请检查数据", part.getPartNo(), part.getPartTypeCode()));
        }

        return partTypePicture;
    }

    private List<MesPartTypeDefect> checkPartTypeDefect(MesPart part, String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(part.getPartTypeCode(), "partTypeCode", packBean);
        List<MesPartTypeDefect> partTypeDefectList = partTypeDefectRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(partTypeDefectList)) {
            throw new ImppBusiException(String.format("未配置【%s】零件类型缺陷类型对应关系，请检查数据", part.getPartTypeCode()));
        }

        return partTypeDefectList;
    }

    private List<MesLocationConfig> checkLocationConfig(String org) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"rowLocation", "columnLocation"}, packBean);
        List<MesLocationConfig> configList = locationConfigRepository.findByHqlWhere(packBean);

        if (CollectionUtils.isEmpty(configList)) {
            throw new ImppBusiException(String.format("未配置位置，请检查数据"));
        }

        return configList;
    }


    /**
     * 构造返回数据
     *
     * @param part
     * @param defectRecordList
     * @param org
     * @return
     */
    private MesNcProcessingModel getModel(MesPart part, List<MesDefectRecord> defectRecordList, String org) {
        //todo 图片暂时放空
        //检查零件类型与图片对应关系
        MesPartTypePicture partTypePicture = checkPartTypePicture(part, org);

        //位置
        List<MesLocationConfig> locationConfigList = checkLocationConfig(org);

        Map<String, List<MesDefectRecord>> defectRecordMap = defectRecordList.stream().collect(Collectors.groupingBy(k -> k.getDefectLocation()));


        Map<String, List<MesDefectRecord>> defectMap = defectRecordList.stream().collect(Collectors.groupingBy(k -> k.getDefectTypeCode()));

        //位置是否标黄Map,key：位置，value：标黄标识
        locationConfigList.forEach(k -> {
            if (defectRecordMap.containsKey(k.getDefectLocation())) {
                k.setIsflg(true);
            }
        });

        List<MesNcProcessingViewModel> defectModelList = new ArrayList<>();
        defectMap.forEach((k, v) -> {
            MesNcProcessingViewModel model = new MesNcProcessingViewModel();
            model.setDefectTypeCode(k);
            model.setQty(v.size());
            model.setDefectTypeName(v.get(0).getDefectTypeName());
            defectModelList.add(model);
        });

        MesNcProcessingModel model = new MesNcProcessingModel();
        model.setDefectModelList(defectModelList);
        model.setPartTypePicture(partTypePicture);
        model.setLocationConfigList(locationConfigList);
        return model;
    }

    /**
     * 保存数据
     *
     * @param part
     * @param defectRecordList
     * @param type
     * @param person
     * @param org
     */
    private void saveDate(MesPart part, List<MesDefectRecord> defectRecordList, Integer type, Integer person, String org) {
        GenSerialNoModel serialNoModel = new GenSerialNoModel("NC_DEFECT_ZRSUM");
        serialNoModel.setPartNo(part.getPartNo());
        ResultBean rb = syncFuncService.syncSerialNo(serialNoModel, AuthUtil.getSessionUser().getUserName(), org, 1);
        String zrsum = "";
        if (null != rb && !CollectionUtils.isEmpty(rb.getResultList())) {
            zrsum = (rb.getResultList().get(0)).toString();
        }

        if (type == MesExtEnumUtil.NC_TYPE.NORMAL.getValue() || type == MesExtEnumUtil.NC_TYPE.RELEASE.getValue()) {
            MesMove move = new MesMove();
            move.setMatnr(part.getPartNo());
            move.setOrganizeCode(org);
            move.setFactoryCode(org);
            move.setLgort(configService.getCfgValue(org, "UMLGO"));
            move.setUmlgo(configService.getCfgValue(org, "LGORT"));
            move.setMenge(defectRecordList.get(0).getQty().intValue());
            move.setMeins(part.getUnit());
            move.setZrsum(zrsum);
            move.setPostDate(TimeTool.getToday());
            move.setPostTime(TimeTool.getTimeShortWithColon());
            ConvertBean.serviceModelInitialize(move, AuthUtil.getSessionUser().getUserName());
            moveRepository.save(move);
        }

        defectRecordList.forEach(k -> {
            k.setNcType(type);
            k.setPerson(person);
            ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
        });

        defectRecordRepository.saveAll(defectRecordList);
    }
}
