package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesJisShippingService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesShippingLoadingCheckService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesShippingLoadingCheckModel;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.MesPcnConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesLoadingList;
import cn.estsh.i3plus.pojo.mes.bean.MesLoadingListDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-04-26 15:38
 * @Modify:
 **/
@Service
@Slf4j
public class MesShippingLoadingCheckService implements IMesShippingLoadingCheckService {

    @Autowired
    private MesLoadingListRepository vehiclesOrderRepository;
    @Autowired
    private MesLoadingListDetailRepository vehiclesOrderDetailRepository;

    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;
    @Autowired
    private MesShippingOrderManagementDetailRepository shippingOrderManagementDetailRepository;
    @Autowired
    private MesProduceSnRepository produceSnRepository;
    @Autowired
    private IMesJisShippingService jisShippingService;
    @Autowired
    private MesJisShippingRepository jisShippingRepository;

    @Override
    public MesShippingLoadingCheckModel queryLoadingOrderNo(MesLoadingList loadingList) {


        //校验装车单
        MesLoadingList loadingListResult = checkLoadingList(loadingList.getOrderCode(), loadingList.getOrganizeCode());


        DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(loadingList.organizeCode);
        DdlPreparedPack.getStringEqualPack(loadingList.getOrderCode(), "orderCode", orderDetailPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"seq"}, orderDetailPackBean);
        List<MesLoadingListDetail> detailList = vehiclesOrderDetailRepository.findByHqlWhere(orderDetailPackBean);

        //是否逆转
        if (loadingListResult.getIsReverse() == MesCommonConstant.TRUE_INTEGER) {
            // 逆转
            for (int i = 0; i < detailList.size(); i += loadingListResult.getReverseQuantity()) {
                if (i + loadingListResult.getReverseQuantity() - 1 < detailList.size()) {
                    Collections.reverse(detailList.subList(i, i + loadingListResult.getReverseQuantity()));
                }
            }
        }

        MesShippingLoadingCheckModel model = new MesShippingLoadingCheckModel();
        Long count = detailList.stream().filter(k -> k.getStatus() == MesExtEnumUtil.LOADING_DETAIL_ORDER_DETAIL_STATUS.SCAN.getValue()).count();
        model.setIsScanCount(count.intValue());
        model.setOrderCode(loadingList.getOrderCode());

        //查询最近扫描完成的装车单号
        DdlPackBean recentlyOrderPackBean = DdlPackBean.getDdlPackBean(loadingList.organizeCode);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.LOADING_ORDER_STATUS.SHIPPING.getValue(), "status", recentlyOrderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"modifyDatetime"}, recentlyOrderPackBean);
        MesLoadingList recentlyVehiclesOrder = vehiclesOrderRepository.getByProperty(recentlyOrderPackBean);

        if (!StringUtils.isEmpty(recentlyVehiclesOrder)) {

            model.setRecentlyScanVehiclesOrder(recentlyVehiclesOrder.getOrderCode());
        }
        model.setDetailList(detailList);

        return model;
    }

    @Override
    public MesLoadingList queryLoadingOrderNoByAutoPushOrder(String shippingGroupNo, String org) {


        //查询最近扫描完成的装车单号
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.LOADING_ORDER_STATUS.CREATE.getValue(), "status", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"seq"}, orderPackBean);
        MesLoadingList loadingList = vehiclesOrderRepository.getByProperty(orderPackBean);

        return loadingList;
    }

    @Override
    public MesShippingLoadingCheckModel saveVehiclesOrderAndDetail(MesShippingLoadingCheckModel model, String org) {

        //校验装车单
        MesLoadingList loadingList = checkLoadingList(model.getOrderCode(), org);

        //校验发运单是否存在
        DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getShippingCode(), "shippingCode", orderDetailPackBean);
        MesLoadingListDetail vehiclesOrderDetail = vehiclesOrderDetailRepository.getByProperty(orderDetailPackBean);
        if (StringUtils.isEmpty(vehiclesOrderDetail)) {
            throw new ImppBusiException(String.format("【%s】此发运单不存在或还未生成装车单明细，请检查数据！", model.getShippingCode()));
        }

        //根据扫描的发运单以及前端传的发运单明细，进行匹配，若没有找到扫描的发运单，则提示扫描的发运单号不在此装车单中，请检查数据！。
        Boolean scanflg = false;
        for (MesLoadingListDetail loadingListDetail : model.getDetailList()) {
            if (loadingListDetail.getShippingCode().equals(model.getShippingCode())) {

                if (loadingListDetail.getTempStatus() == MesCommonConstant.TRUE_INTEGER) {
                    throw new ImppBusiException(String.format("【%s】此发运单已经扫描过，请检查数据！", model.getShippingCode()));
                }
                loadingListDetail.setTempStatus(MesCommonConstant.TRUE_INTEGER);
                loadingListDetail.setTempStatusVal("扫描完成");

                //根据是否启动记忆判断 若启用记忆，则根据保存数据库
                if (loadingList.getIsEnableMemory() == MesCommonConstant.TRUE_INTEGER) {

                    loadingList.setStatus(MesExtEnumUtil.LOADING_ORDER_STATUS.LOADING.getValue());
                    ConvertBean.serviceModelUpdate(loadingList, AuthUtil.getSessionUser().getUserName());
                    vehiclesOrderRepository.save(loadingList);

                    loadingListDetail.setStatus(MesExtEnumUtil.LOADING_DETAIL_ORDER_DETAIL_STATUS.SCAN.getValue());
                    ConvertBean.serviceModelUpdate(loadingListDetail, AuthUtil.getSessionUser().getUserName());
                    vehiclesOrderDetailRepository.save(loadingListDetail);
                }

                //根据扫描的装车单找到是否先装车配置
                //若配置为否，则查询装车单中所有的发运单是否都进行【排序发运校验】完成，若【排序发运校验】没完成，则提示，先进行排序发运校验。
                if (loadingList.getIsFirstInstall() == MesCommonConstant.FALSE_INTEGER) {
                    MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getById(loadingListDetail.getShippingId());
                    if (!StringUtils.isEmpty(orderManagement)) {
                        throw new ImppBusiException(String.format("【%s】此装车单中发运单为【%s】，还未做完【排序发运校验】，请检查数据", model.getShippingCode(), loadingListDetail.getShippingCode()));
                    }
                }
                scanflg = true;
                break;
            }
        }

        if (!scanflg) {
            throw new ImppBusiException(String.format("【%s】扫描的发运单号不在【%s】此装车单中，请检查数据！", model.getShippingCode(), model.getOrderCode()));
        }

        Long count = model.getDetailList().stream().filter(k -> k.getTempStatus() == MesCommonConstant.TRUE_INTEGER).count();
        model.setIsScanCount(count.intValue());

        //全部扫描完成
        if (count.intValue() == model.getDetailList().size()) {

            //装车单主表
            loadingList.setStatus(MesExtEnumUtil.LOADING_ORDER_STATUS.LOADED.getValue());
            ConvertBean.serviceModelUpdate(loadingList, AuthUtil.getSessionUser().getUserName());

            //装车单明细
            DdlPackBean vehiclesOrderDetailPackBean = DdlPackBean.getDdlPackBean(org);
            DdlPreparedPack.getNumEqualPack(loadingList.getId(), "pid", vehiclesOrderDetailPackBean);
            List<MesLoadingListDetail> vehiclesOrderDetailList = vehiclesOrderDetailRepository.findByHqlWhere(vehiclesOrderDetailPackBean);
            vehiclesOrderDetailList.forEach(k -> {
                k.setStatus(MesExtEnumUtil.LOADING_DETAIL_ORDER_STATUS.LOADED.getValue());
                ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            });

            vehiclesOrderDetailRepository.saveAll(vehiclesOrderDetailList);
            vehiclesOrderRepository.save(loadingList);
        }
        return model;
    }

    /**
     * 校验装车单
     *
     * @param orderCode
     * @param org
     */
    public MesLoadingList checkLoadingList(String orderCode, String org) {
//校验装车单是否存在
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(orderCode, "loadingOrderNo", orderPackBean);
        MesLoadingList loadingList = vehiclesOrderRepository.getByProperty(orderPackBean);
        if (StringUtils.isEmpty(loadingList)) {
            throw new ImppBusiException(String.format("【%s】此装车单不存在，请检查数据！", orderCode));
        }

        //校验装车单状态 必须是未装车条码
        if (loadingList.getStatus() != MesExtEnumUtil.LOADING_ORDER_STATUS.CREATE.getValue()) {
            throw new ImppBusiException(String.format("【%s】此装车单状态为【%s】,请选择状态为【%s】状态的装车单", loadingList.getOrderCode(),
                    MesExtEnumUtil.LOADING_ORDER_STATUS.valueOfDescription(loadingList.getStatus()), MesExtEnumUtil.LOADING_ORDER_STATUS.CREATE.getDescription()));
        }
        return loadingList;
    }
}
