package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariableCfg;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Description : 读取加工结果工步
 * @Author : wangjie
 **/
@Slf4j
@Service("mesProductResultReadStepService")
public class MesProductResultReadStepService extends BaseStepService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.getEquipmentVariable(reqBean, MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PRODUCTION.getValue(),
                MesExtEnumUtil.EQUIP_CATEGORY_LEVEL_TWO.SV_OPERATION_MODE.getValue(), MesExtEnumUtil.EQUIP_CATEGORY_LEVEL_TWO.PART_OK.getValue(),
                MesExtEnumUtil.EQUIP_CATEGORY_LEVEL_TWO.PART_NOK.getValue(), MesExtEnumUtil.EQUIP_CATEGORY_LEVEL_TWO.PART_SCRAP.getValue(), MesExtEnumUtil.EQUIP_CATEGORY_LEVEL_TWO.PART_SUSPICIOUS.getValue());

        if (!productionProcessContextStepService.getEquipmentVariableCfg(reqBean, productionProcessContext).getSuccess())
            return execNonCompleteAndSendMsgReturn(reqBean, resultBean, stepResult, productionProcessContext.getMessage());

        if (!productionProcessContextStepService.checkNecessaryEquipmentVariable(reqBean, productionProcessContext, MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PRODUCTION.getValue(), MesExtEnumUtil.EQUIP_CATEGORY_LEVEL_TWO.PART_OK.getValue()).getSuccess())
            return execNonCompleteAndSendMsgReturn(reqBean, resultBean, stepResult, productionProcessContext.getMessage());

        if (!productionProcessContextStepService.checkNecessaryEquipmentVariableCfg(reqBean, productionProcessContext).getSuccess())
            return execNonCompleteAndSendMsgReturn(reqBean, resultBean, stepResult, productionProcessContext.getMessage());

        if (!productionProcessContextStepService.doHandleEquipmentLogListByIds(reqBean, productionProcessContext).getSuccess())
            return execNonCompleteAndSendMsgReturn(reqBean, resultBean, stepResult, productionProcessContext.getMessage());

        MesCellEquipContext cellEquipContext = productionProcessContext.getCurCellEquip();

        Map<String, List<MesEquipmentVariableCfg>> equipmentVariableCfgMap = productionProcessContextStepService.getEquipmentVariableCfgMap(reqBean, productionProcessContext);

        List<MesEquipVariableCollectContext> equipVariableCollectContextList = productionProcessContext.getEquipVariableCfgCollectList();

        //验证加工模式
        Optional<MesEquipVariableCollectContext> optional2Mode = isExistTypeVariableType(equipVariableCollectContextList, MesExtEnumUtil.EQUIP_CATEGORY_LEVEL_TWO.SV_OPERATION_MODE.getValue());
        if (null != optional2Mode && optional2Mode.isPresent() && !checkVariableType2Mode(reqBean, resultBean, stepResult, productionProcessContext, cellEquipContext, equipmentVariableCfgMap, optional2Mode.get()).isCompleted())
            return execNonCompleteAndSendMsgReturn(reqBean, resultBean, stepResult, stepResult.getMsg());

        equipVariableCollectContextList = equipVariableCollectContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getEquipVariableValue()) && !o.getCategoryLevelTwo().equals(MesExtEnumUtil.EQUIP_CATEGORY_LEVEL_TWO.SV_OPERATION_MODE.getValue()))).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(equipVariableCollectContextList)) return execNonCompleteAndSendMsgReturn(reqBean, resultBean, stepResult, String.format("正在持续监听设备[%s]的加工结果...", cellEquipContext.getEquipmentCode()));

        //验证加工结果
        MesEquipmentVariableCfg equipmentVariableCfg = null;
        for (MesEquipVariableCollectContext collectContext : equipVariableCollectContextList) {
            if (null == cellEquipContext) continue;
            equipmentVariableCfg = productionProcessContextStepService.execMatchEquipmentVariableCfgValue(reqBean, productionProcessContext, collectContext, equipmentVariableCfgMap, true);
            if (!productionProcessContext.getSuccess()) this.sendMessage(reqBean, resultBean, productionProcessContext.getMessage(), MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
            productionProcessContext.success();
            if (null != equipmentVariableCfg) break;
        }

        if (null == equipmentVariableCfg) return execNonCompleteAndSendMsgReturn(reqBean, resultBean, stepResult, String.format("获取到设备[%s]数据变量的加工结果值与数据变量接口逻辑信息匹配失败!持续监听中...", cellEquipContext.getEquipmentCode()));

        //TODO 根据二级变量+变量类别 判断结果
        //失败: return false
        //成功： 成功 return true/ 零件报废  return custom
        //可疑/报废: return custom

        return StepResult.getSuccessComplete();
    }

    private StepResult checkVariableType2Mode(StationRequestBean reqBean, StationResultBean resultBean, StepResult stepResult, MesProductionProcessContext productionProcessContext,
                                              MesCellEquipContext cellEquipContext, Map<String, List<MesEquipmentVariableCfg>> equipmentVariableCfgMap, MesEquipVariableCollectContext collectContext) {
        if (StringUtils.isEmpty(collectContext.getEquipVariableValue())) return stepResult.isCompleted(false).msg(String.format("未获取到设备[%s]加工模式数据变量[%s]的结果值,持续监听中...", cellEquipContext.getEquipmentCode(), collectContext.getTagName()));
        MesEquipmentVariableCfg equipmentVariableCfg = productionProcessContextStepService.execMatchEquipmentVariableCfgValue(reqBean, productionProcessContext, collectContext, equipmentVariableCfgMap, true);
        if (!productionProcessContext.getSuccess()) return stepResult.isCompleted(false).msg(productionProcessContext.getMessage());
        if (null == equipmentVariableCfg) return stepResult.isCompleted(false).msg(String.format("获取到设备[%s]加工模式数据变量[%s]的结果值[%s]与数据变量接口逻辑信息匹配失败!持续监听中...", cellEquipContext.getEquipmentCode(), collectContext.getTagName(), collectContext.getEquipVariableValue()));
        return stepResult;
    }

    private Optional<MesEquipVariableCollectContext> isExistTypeVariableType(List<MesEquipVariableCollectContext> equipVariableCollectContextList, String categoryLevelTwo) {
        return equipVariableCollectContextList.stream().filter(o -> (null != o  && o.getCategoryLevelTwo().equals(categoryLevelTwo))).findFirst();
    }

}
