package cn.estsh.i3plus.ext.mes.pcn.pojo.context;

import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariable;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariableCfg;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCell;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCenter;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiParam;
import lombok.Data;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 生产过程上下文对象
 */
@Data
public class MesProductionProcessContext {

    @ApiParam("结果")
    private Boolean success;

    @ApiParam("信息")
    private String message;

    @ApiParam("组织代码")
    private String organizeCode;

    @ApiParam("生产线信息")
    private String workCenterJson;

    @ApiParam("工位信息")
    private String workCellJson;

    @ApiParam("工位设备集合信息")
    private String cellEquipListJson;

    @ApiParam("工序代码")
    private String processCode;

    @ApiParam("工序名称")
    private String processName;

    @ApiParam("工艺代码")
    private String craftCode;

    @ApiParam("工艺代码")
    private String craftName;

    @ApiParam("当前工位设备信息")
    private String curCellEquipJson;

    @ApiParam("设备变量信息")
    private String equipVariableJson;

    @ApiParam("变量接口逻辑信息")
    private String equipVariableCfgJson;

    @ApiParam("变量接口逻辑采集信息")
    private String equipVariableCfgCollectJson;

    @ApiParam("头道模具号")
    private String firstMouldNo;

    @ApiParam("模具号")
    private String mouldNo;

    @ApiParam("一模多腔信息")
    private String mouldCavityJson;

    @ApiParam("腔组明细信息")
    private String cavityGroupDetailJson;

    @ApiParam("设备加工参数信息")
    private String equipProdParamJson;

    @ApiParam("非排序加工规则信息")
    private String unSortRuleJson;

    @ApiParam("装配件信息(非排序/排序线排产装配目视项)")
    private String assemblyJson;

    @ApiParam("时效性信息")
    private String timeEfficientJson;

    @ApiParam("零件条码信息")
    private String produceSnJson;



    public MesProductionProcessContext() {}

    public MesProductionProcessContext(String organizeCode) {
        this.organizeCode = organizeCode;
    }

    public MesProductionProcessContext success() {
        this.success = true;
        this.message = null;
        return this;
    }

    public MesProductionProcessContext message(String message) {
        this.success = false;
        this.message = message;
        return this;
    }

    public MesWorkCenter getWorkCenter() {
        return StringUtils.isEmpty(this.workCenterJson) ? null : JSONObject.parseObject(this.workCenterJson, MesWorkCenter.class);
    }

    public MesProductionProcessContext workCenterJson(MesWorkCenter workCenter) {
        this.workCenterJson = null != workCenter ? JSONObject.toJSONString(workCenter) : null;
        return this;
    }

    public MesWorkCell getWorkCell() {
        return StringUtils.isEmpty(this.workCellJson) ? null : JSONObject.parseObject(this.workCellJson, MesWorkCell.class);
    }

    public MesProductionProcessContext workCellJson(MesWorkCell workCell) {
        this.workCellJson = null != workCell ? JSONObject.toJSONString(workCell) : null;
        return this;
    }

    private List<MesCellEquipContext> getCellEquipList() {
        return StringUtils.isEmpty(this.cellEquipListJson) ? null : JSONObject.parseArray(this.cellEquipListJson, MesCellEquipContext.class);
    }

    public MesProductionProcessContext cellEquipListJson(List<MesCellEquipContext> cellEquipList) {
        this.cellEquipListJson = !CollectionUtils.isEmpty(cellEquipList) ? JSONObject.toJSONString(cellEquipList) : null;
        return this;
    }

    public MesCellEquipContext filterCurCellEquip() {
        List<MesCellEquipContext> cellEquipList = getCellEquipList();
        if (CollectionUtils.isEmpty(cellEquipList) || cellEquipList.size() == 1) return filterCurCellEquip(cellEquipList);
        return filterCurCellEquip(cellEquipList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getSeq()))).sorted(Comparator.comparing(MesCellEquipContext::getSeq)).collect(Collectors.toList()));
    }

    public MesCellEquipContext filterCurCellEquip(String equipmentCode) {
        if (StringUtils.isEmpty(equipmentCode)) return filterCurCellEquip();
        List<MesCellEquipContext> cellEquipList = getCellEquipList();
        return filterCurCellEquip(CollectionUtils.isEmpty(cellEquipList) ? null : cellEquipList.stream().filter(o -> (null != o && o.getEquipmentCode().equals(equipmentCode))).collect(Collectors.toList()));
    }

    private MesCellEquipContext filterCurCellEquip(List<MesCellEquipContext> cellEquipContextList) {
        return CollectionUtils.isEmpty(cellEquipContextList) ? null : cellEquipContextList.get(0);
    }

    public MesCellEquipContext getCurCellEquip() {
        return StringUtils.isEmpty(this.curCellEquipJson) ? null : JSONObject.parseObject(this.curCellEquipJson, MesCellEquipContext.class);
    }

    public MesProductionProcessContext curCellEquipJson(MesCellEquipContext curEquip) {
        this.curCellEquipJson = null != curEquip ? JSONObject.toJSONString(curEquip) : null;
        return this;
    }

    public MesProductionProcessContext processCode(String processCode, String processName) {
        this.processCode = processCode;
        this.processName = processName;
        return this;
    }

    public MesProductionProcessContext craftCode(String craftCode, String craftName) {
        this.craftCode = craftCode;
        this.craftName = craftName;
        return this;
    }

    public List<MesEquipmentVariable> getEquipVariableList() {
        return StringUtils.isEmpty(this.equipVariableJson) ? null : JSONObject.parseArray(this.equipVariableJson, MesEquipmentVariable.class);
    }

    public MesProductionProcessContext equipVariableJson(List<MesEquipmentVariable> equipmentVariableList) {
        if (CollectionUtils.isEmpty(equipmentVariableList)) return this;
        if (!StringUtils.isEmpty(this.equipVariableJson)) equipmentVariableList.addAll(getEquipVariableList());
        this.equipVariableJson = JSONObject.toJSONString(equipmentVariableList);
        return this;
    }

    public List<MesEquipmentVariableCfg> getEquipVariableCfgList() {
        return StringUtils.isEmpty(this.equipVariableCfgJson) ? null : JSONObject.parseArray(this.equipVariableCfgJson, MesEquipmentVariableCfg.class);
    }

    public MesProductionProcessContext equipVariableCfgJson(List<MesEquipmentVariableCfg> equipmentVariableCfgList) {
        if (CollectionUtils.isEmpty(equipmentVariableCfgList)) return this;
        if (!StringUtils.isEmpty(this.equipVariableCfgJson)) equipmentVariableCfgList.addAll(getEquipVariableCfgList());
        this.equipVariableCfgJson = JSONObject.toJSONString(equipmentVariableCfgList);
        return this;
    }

    public List<MesEquipVariableCollectContext> getEquipVariableCfgCollectList() {
        if (StringUtils.isEmpty(this.equipVariableCfgCollectJson)) return null;
        List<MesEquipVariableCollectContext> equipVariableCollectContextList = JSONObject.parseArray(this.equipVariableCfgCollectJson, MesEquipVariableCollectContext.class);
        return equipVariableCollectContextList;
    }

    public MesProductionProcessContext equipVariableCfgCollectJson(List<MesEquipVariableCollectContext> equipVariableCollectContextList) {
        this.equipVariableCfgCollectJson = !CollectionUtils.isEmpty(equipVariableCollectContextList) ? JSONObject.toJSONString(equipVariableCollectContextList) : null;
        return this;
    }


}
